@@ -1,41 +1,40 @@
-use crate::types::*;
+use crate::*;
 use std::fs::OpenOptions;
-use std::io::BufRead;
-use std::io::BufReader;
+use std::io::Error;
+use std::io::Read;
 use std::io::Write;
 use std::net::TcpListener;
-use std::process::exit;
 use std::thread::sleep;
-use std::thread::spawn;
+use std::thread::Builder;
 use std::time::Duration;
 use std::time::Instant;
 
 #[test]
 fn start() {
-  spawn(|| {
-    let listener: TcpListener = TcpListener::bind("127.0.0.1:4567").unwrap_or_else(|_| loop {});
+  Builder::new().name(String::from("Test Web Server")).spawn(|| {
+    let listener: Result<TcpListener, Error> = TcpListener::bind("127.0.0.1:4567");
 
-    let response = "HTTP/1.1 200 OK\r\nContent-Length: 72\r\n\r\n<h1>It Works!</h1><br/><p>If You're Reading This, Then PlayIt Works!</p>";
+    if listener.is_err() {
+      return;
+    }
 
-    for stream in listener.incoming() {
+    let response: &[u8] = "HTTP/1.1 200 OK\r\nContent-Length: 72\r\n\r\n<h1>It Works!</h1><br/><p>If You're Reading This, Then PlayIt Works!</p>".as_bytes();
+
+    for stream in listener.unwrap().incoming() {
       let mut stream = stream.unwrap();
 
-      stream.write(response.as_bytes()).ok();
+      stream.write(response).ok();
       stream.flush().ok();
     }
-  });
+  }).ok();
   let mut playit: PlayIt = PlayIt::new(None).unwrap();
 
-  let tunnel: Tunnel = playit.create_tunnel(CreateTunnelOptions {
-    ip: None,
-    proto: String::from("tcp"),
-    port: 4567,
-  });
+  let tunnel: Tunnel = playit.create_tunnel(4567, Prototype::Tcp, None);
 
   let time: Instant = Instant::now();
   print!("\n\n");
-  println!("Architexture: {}", playit.arch);
-  println!("Operating System: {}", playit.os);
+  println!("Architexture: {:#?}", playit.arch);
+  println!("Operating System: {:#?}", playit.os);
   println!(
     "PlayIt Tunnel: http://{}",
     tunnel.connect_address.as_ref().unwrap()
@@ -45,17 +44,27 @@ fn start() {
   println!("PlayIt Version: {}", playit.version);
   println!("PlayIt Agent Key: {}", playit.agent_key);
   println!("PlayIt Server: {}", playit.server);
-  println!("PlayIt First Lines Of PlayIt Output: \n{}", {
-    let mut command_stderr: String = String::new();
-    let mut command_stdout: String = String::new();
+  println!("PlayIt Output: \n{}\n", {
+    let mut command_stderr = [0; 5000];
+    let mut command_stdout = [0; 5000];
 
-    BufReader::new(playit.playit.stderr.as_mut().unwrap())
-      .read_line(&mut command_stderr)
-      .unwrap();
-    BufReader::new(playit.playit.stdout.as_mut().unwrap())
-      .read_line(&mut command_stdout)
-      .unwrap();
-    [command_stdout, command_stderr].concat()
+    playit
+      .playit
+      .stderr
+      .as_mut()
+      .unwrap()
+      .read(&mut command_stderr)
+      .ok();
+
+    playit
+      .playit
+      .stdout
+      .as_mut()
+      .unwrap()
+      .read(&mut command_stdout)
+      .ok();
+
+    String::from_utf8_lossy(&[command_stdout, command_stderr].concat())
   });
 
   loop {
@@ -82,6 +91,4 @@ fn start() {
   }
 
   sleep(Duration::from_secs(30));
-
-  exit(0)
 }
