@@ -1,18 +1,12 @@
-use crate::consts::*;
-use crate::types::*;
-use reqwest::blocking::Client;
+use crate::*;
+use reqwest::blocking::get;
 use reqwest::blocking::ClientBuilder;
 use reqwest::header;
 use serde_json::from_str;
-use std::any::type_name;
 use std::collections::HashMap;
 use std::env::var;
-use std::fs::create_dir_all;
 use std::fs::read_to_string;
 use std::fs::remove_file;
-use std::fs::File;
-use std::fs::OpenOptions;
-use std::io::Write;
 use std::path::Path;
 use std::path::PathBuf;
 use std::process::Command;
@@ -20,28 +14,18 @@ use std::process::Stdio;
 use url::Url;
 
 pub fn get_defaults(playit_opts: Option<PlayItOpts>) -> Result<PlayIt, String> {
-  let os: String = String::from(if OS == "" {
-    return Err(String::from("Unsupported Operating System"));
-  } else {
-    String::from(OS)
-  });
+  if consts::ARCH == Arch::Unsupported {
+    return Err(String::from("Unsupported Architexture"));
+  }
 
-  let arch: String = String::from({
-    if ARCH == "" {
-      return Err(format!("Unsupported Architexture"));
-    } else {
-      ARCH
-    }
-  });
-
-  let dir: PathBuf = Path::new(&if os == "lin" {
+  let dir: PathBuf = Path::new(&if consts::OS == OS::Linux {
     if var("XDG_CONFIG_HOME").is_err() {
       format!("{}/.config/playit/", var("HOME").unwrap())
     } else {
       format!("{}/playit/", var("XDG_CONFIG_HOME").unwrap())
     }
-  } else if os == "win" {
-    format!("{}/playit/", var("AppData").unwrap())
+  } else if consts::OS == OS::Windows {
+    format!(r"{}\playit\", var("AppData").unwrap())
   } else {
     format!(
       "{}/Library/Application Support/playit/",
@@ -57,142 +41,106 @@ pub fn get_defaults(playit_opts: Option<PlayItOpts>) -> Result<PlayIt, String> {
 
     path
   };
-
-  let destroyed: bool = false;
-  let tunnels: Vec<Tunnel> = Vec::new();
-  let started: bool = false;
-  let used_packets: u8 = 0;
-  let free_packets: u8 = 0;
   let connections: Vec<Connection> = Vec::new();
-  let version: String = String::from(VERSION);
   let download_urls: Binaries = Binaries {
     win: Url::parse(&format!(
       "https://playit.gg/downloads/playit-win_64-{}.exe",
-      version
+      consts::VERSION
     ))
     .expect("Failed To Parse Download URL"),
     lin: Url::parse(&format!(
       "https://playit.gg/downloads/playit-linux_64-{}",
-      version
+      consts::VERSION
     ))
     .expect("Failed To Parse Download URL"),
     mac: Url::parse(&format!(
       "https://playit.gg/downloads/playit-darwin_64-{}.zip",
-      version
+      consts::VERSION
     ))
     .expect("Failed To Parse Download URL"),
     arm64: Url::parse(&format!(
       "https://playit.gg/downloads/playit-aarch64-{}",
-      version
+      consts::VERSION
     ))
     .expect("Failed To Parse Download URL"),
     arm: Url::parse(&format!(
       "https://playit.gg/downloads/playit-armv7-{}",
-      version
+      consts::VERSION
     ))
     .expect("Failed To Parse Download URL"),
   };
-  let binary_type: String = String::from(if os == "lin" && (arch == "arm" || arch == "arm64") {
-    ARCH
-  } else {
-    OS
-  });
-  let binary: PathBuf = download(&dir, &binary_type, &download_urls);
+  let binary: PathBuf = download(&dir, &download_urls).expect("Failed To Download PlayIt");
   let playit: Child = {
     let mut env = HashMap::new();
 
     env.insert(String::from("NO_BROWSER"), String::from("true"));
 
-    if playit_opts.as_ref().is_some() {
+    if let Some(opts) = playit_opts {
       env.insert(
         String::from("PREFERRED_TUNNEL"),
-        playit_opts
-          .clone()
-          .unwrap()
-          .PREFERRED_TUNNEL
-          .unwrap_or(String::from("")),
+        opts.PREFERRED_TUNNEL.unwrap_or_else(|| String::from("")),
       );
       env.insert(
         String::from("PREFERRED_THRESHOLD"),
-        playit_opts
-          .clone()
-          .unwrap()
-          .PREFERRED_THRESHOLD
-          .unwrap_or(50)
-          .to_string(),
+        opts.PREFERRED_THRESHOLD.unwrap_or(50).to_string(),
       );
       env.insert(
         String::from("NO_SPECIAL_LAN"),
-        playit_opts
-          .clone()
-          .unwrap()
-          .NO_SPECIAL_LAN
-          .unwrap_or(false)
-          .to_string(),
+        opts.NO_SPECIAL_LAN.unwrap_or(false).to_string(),
       );
     }
 
     exec(String::from(binary.to_str().unwrap()), None, Some(env)).unwrap()
   };
-  let output: Vec<String> = Vec::new();
-  let stdout: Vec<String> = Vec::new();
-  let stderr: Vec<String> = Vec::new();
-  let errors: Vec<String> = Vec::new();
-  let warnings: Vec<String> = Vec::new();
-
   let (agent_key, server) = loop {
-    let config: Config = from_str::<Config>(
-      &read_to_string(&config_file).unwrap_or(String::from("{}")),
-    )
-    .unwrap_or(Config {
-      agent_key: None,
-      preferred_tunnel: None,
-    });
+    let config: Config =
+      from_str::<Config>(&read_to_string(&config_file).unwrap_or_else(|_| String::from("{}")))
+        .unwrap_or(Config {
+          agent_key: None,
+          preferred_tunnel: None,
+        });
 
     if config.agent_key.is_some() && config.preferred_tunnel.is_some() {
       break (config.agent_key.unwrap(), config.preferred_tunnel.unwrap());
     }
   };
 
-  let req_client: Client = {
-    let mut headers: header::HeaderMap = header::HeaderMap::new();
-    let mut auth: header::HeaderValue =
-      header::HeaderValue::from_str(&format!("agent {}", agent_key)).unwrap();
-    auth.set_sensitive(true);
-    headers.insert(header::AUTHORIZATION, auth);
-    ClientBuilder::new()
-      .default_headers(headers)
-      .build()
-      .unwrap()
-  };
-  let api_path = Url::parse("https://api.playit.gg/").unwrap();
-
-  return Ok(PlayIt {
-    req_client,
-    os,
+  Ok(PlayIt {
+    req_client: {
+      let mut headers: header::HeaderMap = header::HeaderMap::new();
+      let mut auth: header::HeaderValue =
+        header::HeaderValue::from_str(&format!("agent {}", agent_key)).unwrap();
+      auth.set_sensitive(true);
+      headers.insert(header::AUTHORIZATION, auth);
+      ClientBuilder::new()
+        .default_headers(headers)
+        .build()
+        .unwrap()
+    },
+    os: consts::OS,
     config_file,
-    arch,
+    arch: consts::ARCH,
     dir,
-    destroyed,
-    tunnels,
+    destroyed: false,
+    tunnels: Vec::new(),
     agent_key,
-    started,
+    started: false,
     playit,
     server,
-    used_packets,
-    free_packets,
+    used_packets: 0,
+    free_packets: 0,
     connections,
-    version,
+    version: String::from(consts::VERSION),
     download_urls,
     binary,
-    binary_type,
-    output,
-    stdout,
-    stderr,
-    errors,
-    warnings,
-    api_path,
-  });
+    binary_type: consts::BINARY_TYPE,
+    output: Vec::new(),
+    stdout: Vec::new(),
+    stderr: Vec::new(),
+    errors: Vec::new(),
+    warnings: Vec::new(),
+    api_path: Url::parse("https://api.playit.gg/").unwrap(),
+  })
 }
 
 /// Small wrapper for `command`
@@ -203,8 +151,8 @@ pub fn exec(
 ) -> Result<Child, String> {
   let mut command: Command = Command::new(&cmd);
   command
-    .args(args.unwrap_or(Vec::new()))
-    .envs(envs.unwrap_or(HashMap::new()))
+    .args(args.unwrap_or_else(Vec::new))
+    .envs(envs.unwrap_or_else(HashMap::new))
     .stdin(Stdio::piped())
     .stderr(Stdio::piped())
     .stdout(Stdio::piped());
@@ -218,32 +166,37 @@ pub fn exec(
   let command: Child = Child(
     command
       .spawn()
-      .expect(&format!("Failed To Start Process: {}", &cmd)),
+      .unwrap_or_else(|_| panic!("Failed To Start Process: {}", &cmd)),
   );
 
-  return Ok(command);
+  Ok(command)
 }
 
-pub fn trim_newlines(string: &String) -> String {
+pub fn trim_newlines(string: &str) -> String {
   return string.trim_end().replace('\n', " ").replace('\r', "");
 }
 
-pub fn download(dir: &PathBuf, binary_type: &String, download_urls: &Binaries) -> PathBuf {
-  let file: PathBuf = dir
-    .join(&format!(
-      "playit-{}-{}.{}",
-      binary_type,
-      VERSION,
-      String::from(if OS == "win" { "exe" } else { "bin" })
-    ))
-    .to_owned();
+pub fn download(dir: &Path, download_urls: &Binaries) -> Result<PathBuf, String> {
+  let file: PathBuf = dir.join(&format!(
+    "playit-{:#?}-{}.{}",
+    if consts::BINARY_TYPE == BinaryType::Unsupported {
+      return Err(String::from("Unsupported Architexture"));
+    } else {
+      consts::BINARY_TYPE
+    },
+    consts::VERSION,
+    if consts::OS == OS::Windows {
+      "exe"
+    } else {
+      "bin"
+    }
+  ));
 
   if file.exists() {
-    return file;
+    return Ok(file);
   }
-
-  if !file.parent().unwrap().exists() {
-    create_dir_all(file.parent().unwrap()).ok();
+  if !dir.exists() {
+    create_dir_all(dir).ok();
   }
 
   let mut _file: OpenOptions = OpenOptions::new();
@@ -251,7 +204,6 @@ pub fn download(dir: &PathBuf, binary_type: &String, download_urls: &Binaries) -
   _file.create(true).write(true).truncate(true);
 
   #[cfg(any(
-    target_os = "macos",
     target_os = "linux",
     target_os = "freebsd",
     target_os = "dragonfly",
@@ -261,23 +213,33 @@ pub fn download(dir: &PathBuf, binary_type: &String, download_urls: &Binaries) -
   {
     use std::os::unix::fs::OpenOptionsExt;
 
-    _file.mode(0o555);
+    _file.mode(0o777);
   }
 
   let mut _file: File = _file.open(&file).unwrap();
 
   _file
-    .write(
-      &reqwest::blocking::get(download_urls[binary_type].clone())
+    .write_all(
+      &get(download_urls[consts::BINARY_TYPE].clone())
         .unwrap()
         .bytes()
         .expect("Failed To Download PlayIt"),
     )
     .expect("Failed To Download PlayIt");
 
-  return file;
-}
+  #[cfg(target_os = "macos")]
+  {
+    use zip::ZipArchive;
 
-pub fn type_of<T>(_: &T) -> &str {
-  return type_name::<T>();
+    let mut zip: ZipArchive<File> = ZipArchive::new(_file).unwrap();
+
+    zip
+      .extract(dir.join(format!(
+        "playit-{:#?}-{}",
+        consts::BINARY_TYPE,
+        consts::VERSION
+      )))
+      .unwrap();
+  }
+  Ok(file)
 }
